@@ -5,7 +5,7 @@
    
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
+   the Free Software Foundation; either version 3 of the License, or
    (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
@@ -14,8 +14,7 @@
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 #include "includes.h"
@@ -49,7 +48,7 @@ static struct _FAKE_FILE_HANDLE *init_fake_file_handle(enum FAKE_FILE_TYPE type)
 			}
 
 			if ((fh =TALLOC_ZERO_P(mem_ctx, FAKE_FILE_HANDLE))==NULL) {
-				DEBUG(0,("talloc_zero() failed.\n"));
+				DEBUG(0,("TALLOC_ZERO() failed.\n"));
 				talloc_destroy(mem_ctx);
 				return NULL;
 			}
@@ -101,24 +100,27 @@ enum FAKE_FILE_TYPE is_fake_file(const char *fname)
  Open a fake quota file with a share mode.
 ****************************************************************************/
 
-files_struct *open_fake_file(connection_struct *conn,
+NTSTATUS open_fake_file(connection_struct *conn,
 				enum FAKE_FILE_TYPE fake_file_type,
 				const char *fname,
-				uint32 access_mask)
+				uint32 access_mask,
+				files_struct **result)
 {
 	files_struct *fsp = NULL;
+	NTSTATUS status;
 
 	/* access check */
-	if (current_user.uid != 0) {
-		DEBUG(1,("open_fake_file_shared: access_denied to service[%s] file[%s] user[%s]\n",
-			lp_servicename(SNUM(conn)),fname,conn->user));
-		errno = EACCES;
-		return NULL;
+	if (current_user.ut.uid != 0) {
+		DEBUG(3, ("open_fake_file_shared: access_denied to "
+			  "service[%s] file[%s] user[%s]\n",
+			  lp_servicename(SNUM(conn)),fname,conn->user));
+		return NT_STATUS_ACCESS_DENIED;
+
 	}
 
-	fsp = file_new(conn);
-	if(!fsp) {
-		return NULL;
+	status = file_new(conn, &fsp);
+	if(!NT_STATUS_IS_OK(status)) {
+		return status;
 	}
 
 	DEBUG(5,("open_fake_file_shared: fname = %s, FID = %d, access_mask = 0x%x\n",
@@ -128,7 +130,7 @@ files_struct *open_fake_file(connection_struct *conn,
 	fsp->fh->fd = -1;
 	fsp->vuid = current_user.vuid;
 	fsp->fh->pos = -1;
-	fsp->can_lock = True; /* Should this be true ? */
+	fsp->can_lock = False; /* Should this be true ? - No, JRA */
 	fsp->access_mask = access_mask;
 	string_set(&fsp->fsp_name,fname);
 	
@@ -136,11 +138,11 @@ files_struct *open_fake_file(connection_struct *conn,
 	
 	if (fsp->fake_file_handle==NULL) {
 		file_free(fsp);
-		return NULL;
+		return NT_STATUS_NO_MEMORY;
 	}
 
-	conn->num_files_open++;
-	return fsp;
+	*result = fsp;
+	return NT_STATUS_OK;
 }
 
 void destroy_fake_file_handle(FAKE_FILE_HANDLE **fh)
@@ -156,3 +158,9 @@ void destroy_fake_file_handle(FAKE_FILE_HANDLE **fh)
 	talloc_destroy((*fh)->mem_ctx);
 	(*fh) = NULL;
 }
+
+NTSTATUS close_fake_file(files_struct *fsp)
+{
+	file_free(fsp);
+	return NT_STATUS_OK;
+}
